/*  file yasep/core-js/group_ie.js
This file is distributed under terms of the Affero GPL v3 license or later, see http://yasep.org
created 2005-06-11 by whygee@f-cpu.org
2007-04-11 : moved from vsp_eu.js to group_ie.js
2007-04-15 : added the USE_ flags
2007-11-05 : rebranding to YASEP, added support for IMM16_8LSB
2008-11-17 : added OPTIONAL|YASEP32_ONLY|YASEP16_ONLY
2008-11-30 : 16-bit action
2009-01-06 : moved things again to match 16-bit VHDL
    order : MOV SB LZB LSB (16-bits + 32-bits)
            SHH SH LZH LSH (32-bits only)
2009-07-19 : added insert/extract aliases
2009-07-28 : added Y
2009-07-29 : FORM_iR
2009-08-07 : s.snd...
2009-08-09 : IGNORE_SND, IGNORE_SI4
2009-08-10 : Y.NO_WRITEBACK | Y.NO_CONDITION | Y.IGNORE_IMM16
2009-08-23 : removed FORM_X
2011-09-17 : added IMM20

This file implements the "IE" (Insert/Extract) Execution Unit

dependency : group_eu.js and all its parents
*/

Y.prefix=Y.GROUP_IE;

// 16/32-bit instructions :

Y.NewOpcode("MOV",
 function(s) { s.result = s.si4 & F8; },
 function(s) { s.result = s.si4 & F4; },
 Y.FORM_RR | Y.FORM_IR | Y.FORM_iR,
 Y.IGNORE_SND | Y.IGNORE_SND | Y.SWAP_IR |  Y.IMM20,
 "Write imm16/imm20/si4 to dest");

// The Sxx instructions are written with the destination
// (where the byte/word is inserted) last, but it's in fact the 1st field
// so we swap the fields for convenience.
// (2009-01-06 : WTF ?)
Y.NewAlias("IB", "SB",  Y.FORM_RR | Y.FORM_IR | Y.FORM_iR | Y.FORM_iRR | Y.FORM_RRR,
 Y.IMM16_8LSB | Y.ALIAS_IRR | Y.IgnoreImmSign, "Insert a byte into a (half)word");
Y.NewOpcode("SB",
 function(s) {
   var sa=(s.ptr_in & 3) * 8;
   var mask= 0xFF << sa;
   s.result = ((s.si4 << sa) & mask) | (s.snd & ~mask);
//   Y.eu_ptr_out = Y.eu_ptr_in + Y.eu_ptr_wr_dir;
 },
 function(s) {
   var sa=(s.ptr_in & 1) * 8;
   var mask= 0xFF << sa;
   s.result = ((s.si4 << sa) & mask) | (s.snd & ~mask);
//   s.ptr_out = Y.eu_ptr_in +  Y.eu_ptr_wr_dir;
 },
 Y.FORM_RR | Y.FORM_IR | Y.FORM_iR | Y.FORM_iRR | Y.FORM_RRR,
 Y.IMM16_8LSB | Y.ALIAS_IRR | Y.IgnoreImmSign,
 "Store the imm16/si4 Byte");


Y.NewAlias("EZB","LZB", Y.FORM_RR,
 Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Extract and Zero-extend Byte");
Y.NewOpcode("LZB",
 function(s) {
   s.result = 0xFF & (s.snd >> ((s.ptr_in & 3) * 8));
//   s.ptr_out = Y.eu_ptr_in + Y.eu_ptr_rd_dir;
 },
 function(s) {
   s.result = 0xFF & (s.snd >> ((s.ptr_in & 1) * 8));
//   s.ptr_out = Y.eu_ptr_in + Y.eu_ptr_rd_dir;
 },
 Y.FORM_RR, Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Load and Zero-extend Byte");

Y.NewAlias("ESB","LSB", Y.FORM_RR,
 Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Extract and Sign-extend Byte");
Y.NewOpcode("LSB",
 function(s) {
   s.result = 0xFF & (s.snd >> ((s.ptr_in & 3) * 8));
   if (s.result & 0x80)
     s.result |= 0xFFFFFF00;
//   s.ptr_out = Y.eu_ptr_in + Y.eu_ptr_rd_dir;
 },
 function(s) {
   s.result = 0xFF & (s.snd >> ((s.ptr_in & 1) * 8));
   if (s.result & 0x80)
     s.result |= 0xFF00;
//   Y.eu_ptr_out = Y.eu_ptr_in + Y.eu_ptr_rd_dir;
 },
 Y.FORM_RR, Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Load and Sign-extend Byte");

/////////////////////////////////////////////////////////
// 32-bit instructions :

Y.NewAlias("IH","SH", Y.FORM_RR |  Y.FORM_IR | Y.FORM_IR | Y.FORM_iRR | Y.FORM_iRR | Y.FORM_RRR,
 Y.YASEP32_ONLY, "Insert the imm16/si4 Half-word");
Y.NewOpcode("SH",
 function(s) {
   if ((s.ptr_in & 3) > 2) {
     s.msg("orange", "Error : "+'IE.SH'+" : out of word access");
//     s.ptr_out=null;
     s.result=null;
   }
   else {
     var sa=(s.ptr_in & 3) * 8;
     var mask= F4 << sa;
     s.result = ((s.si4<<sa) & mask) | (s.snd & ~mask);
//     s.ptr_out = Y.eu_ptr_in + (Y.eu_ptr_wr_dir*2);
   }
 },
 null, // YASEP32_ONLY
 Y.FORM_RR | Y.FORM_IR | Y.FORM_IRR | Y.FORM_iR | Y.FORM_iRR | Y.FORM_RRR,
 Y.YASEP32_ONLY | Y.ALIAS_IRR,
 "Store the imm16/si4 Half-word");

Y.NewAlias("IHH","SHH", Y.FORM_RR | Y.FORM_IR | Y.FORM_IRR | Y.FORM_iR | Y.FORM_iRR | Y.FORM_RRR,
 Y.YASEP32_ONLY | Y.ALIAS_IRR,
 "Insert the imm16/si4 Half-word High");
Y.NewOpcode("SHH",
 function(s) {
   s.result = (s.si4 & F4)<<16 | (s.snd & F4);
 },
 null, // YASEP32_ONLY
 Y.FORM_RR | Y.FORM_IR | Y.FORM_IRR | Y.FORM_iR | Y.FORM_iRR | Y.FORM_RRR,
 Y.YASEP32_ONLY | Y.ALIAS_IRR,
 "Store the imm16/si4 Half-word High");

Y.NewAlias("EZH","LZH", Y.FORM_RR,
 Y.YASEP32_ONLY | Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Extract and Zero-extend Half-word");
Y.NewOpcode("LZH",
 function(s) {
   if ((s.ptr_in & 3) > 2) {
     s.msg("orange", "Error : "+'IE.LZH'+" : out of word access");
//     s.ptr_out=null;
     s.result=null;
   }
   else {
     s.result = F4 & (s.snd >> ((s.ptr_in & 3) * 8));
//     Y.eu_ptr_out = Y.eu_ptr_in + (Y.eu_ptr_rd_dir*2);
   }
 },
 null, // YASEP32_ONLY
 Y.FORM_RR, Y.YASEP32_ONLY | Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Load and Zero-extend Half-word");


Y.NewAlias("ESH","LSH", Y.FORM_RR,
 Y.YASEP32_ONLY | Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Extract and Sign-extend Half-word");
Y.NewOpcode("LSH",
 function(s) {
   if ((s.ptr_in & 3) > 2) {
     s.msg("orange", "Error : "+'IE.LSH'+" : out of word access");
//     s.ptr_out=null;
     s.result=null;
   }
   else {
     s.result = F4 & (s.snd >> ((s.ptr_in & 3) * 8));
     if (s.result & 0x80)
       s.result |= F4<<16; // 0xFFFF0000;
//     s.ptr_out = Y.eu_ptr_in + (Y.eu_ptr_rd_dir*2);
   }
 },
 null, // YASEP32_ONLY
 Y.FORM_RR, Y.YASEP32_ONLY | Y.IGNORE_SI4 | Y.IGNORE_IMM16,
 "Load and Sign-extend Half-word");
