/*yasep/cookies.js
This file is distributed under terms of the Affero GPL v3 license or later, see http://yasep.org
created 20110808 by whygee@f-cpu.org
version mar. nov. 22 23:03:02 CET 2011
depends on core-js/base66s.js

this file manages the persistent user interface parameters.
a "cache" is maintained, numbers are encoded
in base66s. LANG.LANG is updated if a cookie already exists.

in the future : windows are stored as .name-xpos-ypos-xsize-ysize[-eventual options like hash]
==> name is a number (the order of the key), negative if the window is hidden

cookies_cache[0]= site version
cookies_cache[1]= language, 1=english, 2 french...

20120902 : the cookie's name is "a=", followed by the site's version number.
This way, the keys can be reduced to one number,
and the keys can change without polluting the cookies' meanings.
*/

init_obj.push(cookies={
  cache:[],
  cache_string:"",
  first_run:true, // No cookie found (yet) so it must be a first run

  filterRegExp:new RegExp("[^"+baseLUT+"]","g"), // filter out any character that is NOT in the base66 domain

  set:function( val, expires, path, domain, secure ) {
    var today = new Date();
    today.setTime( today.getTime() );
    if (expires)
      expires = expires * 86400000; // * 1000 * 60 * 60 * 24;
    var expires_date = new Date( today.getTime() + (expires) );
    document.cookie = "a="+escape(val) +
      ((expires) ? ";expires="+expires_date.toGMTString() : "" )+
      ((path)    ? ";path=" + path : "" ) +
      ((domain)  ? ";domain=" + domain : "" ) +
      ((secure)  ? ";secure" : "" );
  },

///////////////////////////////////////////////////////////////
// build the megacookie
  save:function(s){ // s comes from gui-js/winman.js:dumpWinCache
    if (typeof s=="string")
      cookies.cache_string=s; // update cache
    else
      s=cookies.cache_string; // read cache

    cookies.cache[0]=""+VERSION;
    cookies.cache[1]=""+encode_number_base66(LANG.LANG+1);
//    if (FF.target_loopback)
//      cookies.cache[2]=""+FF.target_loopback.replace(/[.]/g,"%2E");
//    else

    var c=cookies.cache; //.slice(0,2);
    if (c.length <= 2) // has the cache been truncated by body_start() ?
      cookies.set(c.join(".")+s,10); // expiry after 10 days
      // (so you don't get the "first steps" window too often)
  },

///////////////////////////////////////////////////////////////
// manage the YGWM megacookie :
  init_obj: function(){
    var c=unescape(document.cookie), end;

    if (!c // cookie absent
    || (c[0]!="a") || (c[1]!="=")) // not our own cookie at the start of the line ? o_O
      return (cookies.cache=[]); // leaves cookies.first_run==true

    if ((end = c.indexOf(";", 2)) == -1)
         end = c.length;

    c=cookies.cache
     =c.substring(2,end).split(".");

    if (c[0]!=VERSION)  // valid condition implies/includes c[0] existant
         // assert: VERSION!=null
      return (cookies.cache=[]); // does not really return the array but it's smaller /o\
       // but we have to flush the cookies if the date does not match, or else the key numbers may not match

    cookies.first_run=false; // ok we now know this site has already been loaded.

    if (c[1]) {
//      // Trap : "" means 0 in base66 but it also means absent... so I shift/increment the value.
//      // "" will mean nothing yet, "1" means 0=english, "2"=french...
      LANG.LANG=decode_number_base66(c[1]+"")-1;  // make sure it's a string
    }
//    if (cookies.cache[1] && (cookies.cache[1]!=""))
//      FF.target_loopback=cookies.cache[1].replace(/[%]2E/g,".");
  }
});
